@@ -1,8 +1,8 @@
 /*
- * Copyright 2004-2007  Luc Verhaegen <lverhaegen@novell.com>
- * Copyright 2007       Matthias Hopf <mhopf@novell.com>
- * Copyright 2007       Egbert Eich   <eich@novell.com>
- * Copyright 2007  Advanced Micro Devices, Inc.
+ * Copyright 2004-2008  Luc Verhaegen <lverhaegen@novell.com>
+ * Copyright 2007, 2008 Matthias Hopf <mhopf@novell.com>
+ * Copyright 2007, 2008 Egbert Eich   <eich@novell.com>
+ * Copyright 2007, 2008 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,7 +30,7 @@
 #include "xf86.h"
 #include "xf86DDC.h"
 #if HAVE_XF86_ANSIC_H
-# include "xf86_ansic.h" 
+# include "xf86_ansic.h"
 #else
 # include <stdio.h>
 # include <string.h>
@@ -44,6 +44,9 @@
 #include "rhd_modes.h"
 #include "rhd_monitor.h"
 
+/* For Acceleration FB validation */
+#include "r5xx_accel.h"
+
 /*
  * Define a set of own mode errors.
  */
@@ -532,10 +535,6 @@ rhdModeSanity(DisplayModePtr Mode)
     if ((Mode->VScan != 0) && (Mode->VScan != 1))
         return MODE_NO_VSCAN;
 
-    /* should be possible, but is untested */
-    if (Mode->Flags & V_INTERLACE)
-        return MODE_NO_INTERLACE;
-
     if (Mode->Flags & V_DBLSCAN)
         return MODE_NO_DBLESCAN;
 
@@ -602,6 +601,10 @@ rhdModeFillOutCrtcValues(DisplayModePtr Mode)
     Mode->HSync = ((float) Mode->SynthClock) / Mode->CrtcHTotal;
     Mode->VRefresh = (Mode->SynthClock * 1000.0) /
         (Mode->CrtcHTotal * Mode->CrtcVTotal);
+    if (Mode->Flags & V_INTERLACE)
+	Mode->VRefresh *= 2.0;
+    if (Mode->Flags & V_DBLSCAN)
+	Mode->VRefresh /= 2.0;
 
     /* We're usually first in the chain, right after rhdModeSanity. */
     Mode->CrtcHAdjusted = FALSE;
@@ -772,8 +775,8 @@ rhdModeValidateCrtc(struct rhdCrtc *Crtc, DisplayModePtr Mode)
             continue;
 
 	Status = Crtc->FBValid(Crtc, Mode->CrtcHDisplay, Mode->CrtcVDisplay,
-			       pScrn->bitsPerPixel, rhdPtr->FbFreeStart,
-			       rhdPtr->FbFreeSize, NULL);
+			       pScrn->bitsPerPixel, rhdPtr->FbScanoutStart,
+			       rhdPtr->FbScanoutSize, NULL);
         if (Status != MODE_OK)
             return Status;
 
@@ -783,11 +786,21 @@ rhdModeValidateCrtc(struct rhdCrtc *Crtc, DisplayModePtr Mode)
         if (Mode->CrtcHAdjusted || Mode->CrtcVAdjusted)
             continue;
 
-	Status = Crtc->PLL->Valid(Crtc->PLL, Mode->Clock);
-        if (Status != MODE_OK)
-            return Status;
-        if (Mode->CrtcHAdjusted || Mode->CrtcVAdjusted)
-            continue;
+	if (Crtc->ScaleValid) {
+	    Status = Crtc->ScaleValid(Crtc, RHD_CRTC_SCALE_TYPE_NONE, Mode, NULL);
+	    if (Status != MODE_OK)
+		return Status;
+	    if (Mode->CrtcHAdjusted || Mode->CrtcVAdjusted)
+		continue;
+	}
+
+	if (Crtc->PLL->Valid) {
+	    Status = Crtc->PLL->Valid(Crtc->PLL, Mode->Clock);
+	    if (Status != MODE_OK)
+		return Status;
+	    if (Mode->CrtcHAdjusted || Mode->CrtcVAdjusted)
+		continue;
+	}
 
         for (Output = rhdPtr->Outputs; Output; Output = Output->Next)
             if (Output->Active && (Output->Crtc == Crtc)) {
@@ -1372,28 +1385,43 @@ RHDGetVirtualFromConfig(ScrnInfoPtr pScrn)
     CARD32 VirtualY = pScrn->display->virtualY;
     CARD32 Pitch1, Pitch2;
     float Ratio = (float) pScrn->display->virtualY / pScrn->display->virtualX;
-    int ret1 = FALSE, ret2 = FALSE;
+    int ret = FALSE;
 
     RHDFUNC(pScrn);
 
     while (VirtualX && VirtualY) {
-	ret1 = Crtc1->FBValid(Crtc1, VirtualX, VirtualY, pScrn->bitsPerPixel,
-			      rhdPtr->FbFreeStart, rhdPtr->FbFreeSize, &Pitch1);
-	ret2 = Crtc2->FBValid(Crtc2, VirtualX, VirtualY, pScrn->bitsPerPixel,
-			      rhdPtr->FbFreeStart, rhdPtr->FbFreeSize, &Pitch2);
+	ret = Crtc1->FBValid(Crtc1, VirtualX, VirtualY, pScrn->bitsPerPixel,
+			     rhdPtr->FbScanoutStart, rhdPtr->FbScanoutSize, &Pitch1);
+	if (ret != MODE_OK)
+	    goto shrink;
+	ret = Crtc2->FBValid(Crtc2, VirtualX, VirtualY, pScrn->bitsPerPixel,
+			     rhdPtr->FbScanoutStart, rhdPtr->FbScanoutSize, &Pitch2);
+	if (ret != MODE_OK)
+	    goto shrink;
 
-	if ((ret1 == MODE_OK) && (ret2 == MODE_OK) && (Pitch1 == Pitch2)) {
-	    pScrn->virtualX = VirtualX;
-	    pScrn->virtualY = VirtualY;
-	    pScrn->displayWidth = Pitch1;
-	    return TRUE;
-	}
+	if (Pitch1 != Pitch2)
+	    goto shrink;
 
+	/* let 2d acceleration have a say as well */
+	if (rhdPtr->AccelMethod >= RHD_ACCEL_XAA)
+	    if (rhdPtr->ChipSet < RHD_R600) /* badly abstracted, i know */
+		if (!R5xx2DFBValid(rhdPtr, VirtualX, VirtualY, pScrn->bitsPerPixel,
+				   rhdPtr->FbScanoutStart, rhdPtr->FbScanoutSize, Pitch1))
+		    goto shrink;
+
+	break; /* must be good then. */
+    shrink:
 	VirtualX--;
 	VirtualY = Ratio * VirtualX;
     }
 
-    return FALSE;
+    if (VirtualX && VirtualY) {
+	pScrn->virtualX = VirtualX;
+	pScrn->virtualY = VirtualY;
+	pScrn->displayWidth = Pitch1;
+	return TRUE;
+    } else
+	return FALSE;
 }
 
 /*
@@ -1408,7 +1436,7 @@ RHDGetVirtualFromModesAndFilter(ScrnInfoPtr pScrn, DisplayModePtr Modes, Bool Si
     CARD32 VirtualX = 0;
     CARD32 VirtualY = 0;
     CARD32 Pitch1, Pitch2;
-    int ret1 = FALSE, ret2 = FALSE;
+    int ret = FALSE;
 
     RHDFUNC(pScrn);
 
@@ -1431,40 +1459,51 @@ RHDGetVirtualFromModesAndFilter(ScrnInfoPtr pScrn, DisplayModePtr Modes, Bool Si
 	    else
 		VirtualY = pScrn->virtualY;
 
-	    ret1 = Crtc1->FBValid(Crtc1, VirtualX, VirtualY, pScrn->bitsPerPixel,
-				  rhdPtr->FbFreeStart, rhdPtr->FbFreeSize, &Pitch1);
-	    ret2 = Crtc2->FBValid(Crtc2, VirtualX, VirtualY, pScrn->bitsPerPixel,
-				  rhdPtr->FbFreeStart, rhdPtr->FbFreeSize, &Pitch2);
-
-	    if ((ret1 == MODE_OK) && (ret2 == MODE_OK) && (Pitch1 == Pitch2)) {
-		Mode = Mode->next;
-		pScrn->virtualX = VirtualX;
-		pScrn->virtualY = VirtualY;
-		pScrn->displayWidth = Pitch1;
-	    } else {
-		if (!Silent) {
-		    if (ret1 != MODE_OK)
-			xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Rejected mode \"%s\" "
-				   "(%dx%d): %s\n", Mode->name,
-				   Mode->HDisplay, Mode->VDisplay,
-				   rhdModeStatusToString(ret1));
-		    else if (ret2 != MODE_OK)
-			xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Rejected mode \"%s\" "
-				   "(%dx%d): %s\n", Mode->name,
-				   Mode->HDisplay, Mode->VDisplay,
-				   rhdModeStatusToString(ret2));
-		    else {
-			xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Rejected mode \"%s\" "
-				   "(%dx%d): %s\n", Mode->name,
-				   Mode->HDisplay, Mode->VDisplay,
-				   "CRTC Pitches do not match");
-		    }
-		}
-
-		Next = Mode->next;
-		Modes = rhdModeDelete(Modes, Mode);
-		Mode = Next;
+	    /* Check what Crtc1 thinks this should be. */
+	    ret = Crtc1->FBValid(Crtc1, VirtualX, VirtualY, pScrn->bitsPerPixel,
+				 rhdPtr->FbScanoutStart, rhdPtr->FbScanoutSize, &Pitch1);
+	    if (ret != MODE_OK) {
+		xf86DrvMsg(pScrn->scrnIndex, X_INFO, "%s rejected mode \"%s\" "
+			   "(%dx%d): %s\n", Crtc1->Name, Mode->name,
+			   Mode->HDisplay, Mode->VDisplay,
+			   rhdModeStatusToString(ret));
+		goto rejected;
 	    }
+
+	    /* Check what Crtc2 thinks this should be. */
+	    ret = Crtc2->FBValid(Crtc2, VirtualX, VirtualY, pScrn->bitsPerPixel,
+				 rhdPtr->FbScanoutStart, rhdPtr->FbScanoutSize, &Pitch2);
+	    if (ret != MODE_OK) {
+		xf86DrvMsg(pScrn->scrnIndex, X_INFO, "%s rejected mode \"%s\" "
+			   "(%dx%d): %s\n", Crtc2->Name, Mode->name,
+			   Mode->HDisplay, Mode->VDisplay,
+			   rhdModeStatusToString(ret));
+		goto rejected;
+	    }
+
+	    /* when needed, check whether this matches our 2D engine as well. */
+	    if (rhdPtr->AccelMethod >= RHD_ACCEL_XAA)
+		if (rhdPtr->ChipSet < RHD_R600) /* badly abstracted, i know */
+		    if (!R5xx2DFBValid(rhdPtr, VirtualX, VirtualY,
+				       pScrn->bitsPerPixel, rhdPtr->FbScanoutStart,
+				       rhdPtr->FbScanoutSize, Pitch1)) {
+			xf86DrvMsg(pScrn->scrnIndex, X_INFO, "2D acceleration "
+				   "rejected mode \"%s\" (%dx%d).\n",
+				   Mode->name, Mode->HDisplay, Mode->VDisplay);
+			goto rejected;
+		    }
+
+	    /* mode is perfectly valid FB wise */
+	    Mode = Mode->next;
+	    pScrn->virtualX = VirtualX;
+	    pScrn->virtualY = VirtualY;
+	    pScrn->displayWidth = Pitch1;
+	    continue;
+
+	rejected:
+	    Next = Mode->next;
+	    Modes = rhdModeDelete(Modes, Mode);
+	    Mode = Next;
 	} else
 	    Mode = Mode->next;
     }
@@ -1507,8 +1546,8 @@ RHDRRModeFixup(ScrnInfoPtr pScrn, DisplayModePtr Mode, struct rhdCrtc *Crtc,
 	if (Crtc) {
 	    /* Check FB */
 	    Status = Crtc->FBValid(Crtc, Mode->CrtcHDisplay, Mode->CrtcVDisplay,
-				   pScrn->bitsPerPixel, rhdPtr->FbFreeStart,
-				   rhdPtr->FbFreeSize, NULL);
+				   pScrn->bitsPerPixel, rhdPtr->FbScanoutStart,
+				   rhdPtr->FbScanoutSize, NULL);
 	    if (Status != MODE_OK)
 		return Status;
 
@@ -1519,12 +1558,22 @@ RHDRRModeFixup(ScrnInfoPtr pScrn, DisplayModePtr Mode, struct rhdCrtc *Crtc,
 	    if (Mode->CrtcHAdjusted || Mode->CrtcVAdjusted)
 		continue;
 
+	    if (Crtc->ScaleValid) {
+		Status = Crtc->ScaleValid(Crtc, RHD_CRTC_SCALE_TYPE_NONE, Mode, NULL);
+		if (Status != MODE_OK)
+		    return Status;
+		if (Mode->CrtcHAdjusted || Mode->CrtcVAdjusted)
+		    continue;
+	    }
+
 	    /* Check PLL */
-	    Status = Crtc->PLL->Valid(Crtc->PLL, Mode->Clock);
-	    if (Status != MODE_OK)
-		return Status;
-	    if (Mode->CrtcHAdjusted || Mode->CrtcVAdjusted)
-		continue;
+	    if (Crtc->PLL->Valid) {
+		Status = Crtc->PLL->Valid(Crtc->PLL, Mode->Clock);
+		if (Status != MODE_OK)
+		    return Status;
+		if (Mode->CrtcHAdjusted || Mode->CrtcVAdjusted)
+		    continue;
+	    }
 	}
 
 	/* Check Output */
